// BezierView.cpp : implementation of the CBezierView class
//
/*

  Title:	Bezier Curve
  Author:	Claudio Eduardo Storti Lago
  email:	webmaster@computersorcerer.com
  http://www.computersorcerer.com

  ps:	this source can be distributed freely, but you need to
		mention the author's name.

*/

#include "stdafx.h"
#include "Bezier.h"

#include "BezierDoc.h"
#include "BezierView.h"
#include "NumPointsDlg.h"
#include "ResolutionDlg.h"

#include "Stack.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBezierView

IMPLEMENT_DYNCREATE(CBezierView, CView)

BEGIN_MESSAGE_MAP(CBezierView, CView)
	//{{AFX_MSG_MAP(CBezierView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_OPTIONS_NUMBEROFPOINTS, OnOptionsNumberofpoints)
	ON_COMMAND(ID_OPTIONS_REDRAW, OnOptionsRedraw)
	ON_COMMAND(ID_OPTIONS_RESET, OnOptionsReset)
	ON_COMMAND(ID_OPTIONS_RESOLUTION, OnOptionsResolution)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBezierView construction/destruction

CBezierView::CBezierView()
{
	// TODO: add construction code here
	m_centerX=0;
	m_centerY=0;
	m_resolution=200;
	m_stack.m_maxPoints=100;
}

CBezierView::~CBezierView()
{
}

BOOL CBezierView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBezierView drawing

void CBezierView::OnDraw(CDC* pDC)
{
	CBezierDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	point2d_ptr aux;
	
	CRect rcBounds;
	CPen LightGray;
	CPen BluePen;
	CPen RedPen;
	float nx,ny;
	short index;

	GetWindowRect(&rcBounds);

	int centerX=rcBounds.Width()/2;
	int centerY=rcBounds.Height()/2;
	
	m_centerX=centerX;
	m_centerY=centerY;

	//desenha os eixos
	LightGray.CreatePen(PS_SOLID,0,RGB(205,205,205));
	pDC->SelectObject(LightGray);
	pDC->MoveTo(centerX,0);
	pDC->LineTo(centerX,rcBounds.Height());
	pDC->MoveTo(0,centerY);
	pDC->LineTo(rcBounds.Width(),centerY);

	//Cria a caneta azul light e desenha a linha fornecida
	BluePen.CreatePen(PS_SOLID,0,RGB(200,200,255));
	pDC->SelectObject(BluePen);

	if (m_stack.GetNumPoints()>1) {
		pDC->MoveTo(m_stack.points->x+centerX,centerY-m_stack.points->y);
		aux=m_stack.points->next;
		while(aux!=NULL){
			pDC->LineTo(aux->x+centerX,centerY-aux->y);
			aux=aux->next;
		}
	}

	//Cria a caneta vermelho forte e desenha a curva de Bzier
	RedPen.CreatePen(PS_SOLID,0,RGB(255,0,0));
	pDC->SelectObject(RedPen);

	//Move para o primeiro ponto
	if (m_resolution>0 && m_stack.GetNumPoints()>1){
		BezierPoint(0,&nx,&ny);
		pDC->MoveTo(centerX+(int)nx,centerY-(int)ny);
		//Desenha os segmentos
		for (index=1;index<=m_resolution;index++){
			BezierPoint(index,&nx,&ny);
			pDC->LineTo(centerX+(int)nx,centerY-(int)ny);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBezierView diagnostics

#ifdef _DEBUG
void CBezierView::AssertValid() const
{
	CView::AssertValid();
}

void CBezierView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBezierDoc* CBezierView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBezierDoc)));
	return (CBezierDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBezierView message handlers

void CBezierView::BezierPoint(int t, float * x, float * y)
{
	float serie, resto;
	long index;
	float nx=0, ny=0, nt=(float)t, np;
	point2d_ptr aux=m_stack.points;

	serie=1;
	for (index=0;index<m_stack.GetNumPoints();index++){
		resto=pot(nt/m_resolution,index)*pot(1-(nt/m_resolution),m_stack.GetNumPoints()-1-index);
		nx+=serie*resto*aux->x;
		ny+=serie*resto*aux->y;
		np=(float)m_stack.GetNumPoints();
		serie*=(np-1-index)/(index+1);
		aux=aux->next;
	}
	*x=nx;
	*y=ny;
}

float CBezierView::pot(float a, int b)
{
	int index;
	float resultado;

	if (b==0)
		return 1;
	else
		if (b==1)
			return a;
		else {
			resultado=a;
			for (index=0;index<b-1;index++)
				resultado*=a;
		}

	return resultado;
}

void CBezierView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_stack.AddPoint(point.x-m_centerX,(point.y-m_centerY)*-1);
	Invalidate();
	CView::OnLButtonDown(nFlags, point);
}

void CBezierView::OnOptionsNumberofpoints() 
{
	// TODO: Add your command handler code here
	CNumPointsDlg dlg;

	dlg.m_nPoints=m_stack.m_maxPoints;
	UpdateData(false);
	if (dlg.DoModal()==IDOK){
		UpdateData(true);
		m_stack.m_maxPoints=dlg.m_nPoints;
	}
	
}

void CBezierView::OnOptionsRedraw() 
{
	// TODO: Add your command handler code here
	Invalidate();
}

void CBezierView::OnOptionsReset() 
{
	// TODO: Add your command handler code here
	m_stack.CleanAll();
}

void CBezierView::OnOptionsResolution() 
{
	// TODO: Add your command handler code here
	CResolutionDlg dlg;

	dlg.m_resolution=m_resolution;
	UpdateData(false);
	if (dlg.DoModal()==IDOK){
		UpdateData(true);
		m_resolution=dlg.m_resolution;
	}
	Invalidate();
}
